function formatDate(date) {
    var dd = date.getDate()
    if ( dd < 10 ) dd = '0' + dd;
    var mm = date.getMonth()+1
    if ( mm < 10 ) mm = '0' + mm;
    var yy = date.getFullYear();
    if ( yy < 10 ) yy = '0' + yy;
    return dd+'.'+mm+'.'+yy;
}

function sensor(id, name, unit, unit2) {
    var e = document.createElement('li');
    var link = document.createElement('a');
    e.setAttribute('id', id);
    e.setAttribute('unit', unit);
    e.setAttribute('title', name);
    if (unit2 !== undefined) 
        e.setAttribute('unit2', unit2);
    link.setAttribute('href', '#');
    link.innerHTML = name;
    e.appendChild(link);
    return e;
}

function updateData(script, id ,date) {
    $.ajax({
        url: 'scripts/'+script + '?id='+id + '&date='+date,
        method: 'post',
    });
}

function makePlotD1(event) {
    var panelname = $(this).parents('div .ui-widget-content').attr('id');
    var timekey = $('#' + panelname + ' input[type=radio]:checked').val();
    var id = $(this).attr('id');
    var title = $(this).attr('title');
    console.log(title);
    var targetdiv = event.data.target;
    var targetdivwidth = parseFloat($(targetdiv).css('width'));
    var unit = $(this).attr('unit');


    $.ajax({
        url: 'cache/' + panelname + '.' + id + '.' + timekey + '.json',
        method: 'post',
        dataType: 'json',
        data: {timekey: timekey, id: id},
        success: plotdata
    });

    function plotdata(data) {
        var dataset = [];
        var rows = data['data'].length;

        var factor = Math.floor(data['time'].length/targetdivwidth/3 + 0.5);
        if (factor == 0) factor = 1;
        console.log(factor);

        for (var w = 0; w < rows; w++) {
          var temp = [];
          for (var i = 0; i < data['time'].length; i+=factor) {
              temp.push([data['time'][i]*1000, data['data'][w][i]]);
          }
          dataset.push(temp);
        }

        var d0 = new Date(data['time'][0]*1000);
        var dmin = Date.UTC(d0.getUTCFullYear(), d0.getUTCMonth(), d0.getUTCDate(), 0, 0, 0);
        var dmax = dmin + 86400000;
        var hour = 3600000; 
        var ticks = [];
        for (var i=0; i<=24; i+=3) ticks.push(dmin + i*hour);

        var options = {
            canvas: true,
            xaxis: { 
                mode: "time",
                ticks: ticks,
                min: dmin,
                max: dmax,
                timeformat: "%H:%M",
                axisLabelUseCanvas: true,
                axisLabelPadding: 16,
                axisLabelFontSizePixels: 16,
                axisLabelFontFamily: 'Arial',
                axisLabelColour: 'black',
                axisLabel: 'UTC Time   on   ' + formatDate(new Date(dmin)),
            },
            yaxis: {
                labelWidth: 40,
                axisLabelUseCanvas: true,
                axisLabelFontFamily: 'Arial',
                axisLabel: '[ ' + unit + ' ]'
            },
            series: {
                shadowSize: 0,
            },
        }

        var newplot = $.plot(targetdiv, dataset, options);

        /*
        var divholder = document.createElement('div');
        var divelement = document.createElement('div');
        divelement.style.width=600+'px';
        divelement.style.height=300+'px';
        var title = document.createElement('p');
        title.innerHTML = 'abc';
        divholder.appendChild(title);
        divholder.appendChild(divelement);
        var printversion = $.plot(divelement, dataset, options);
        console.log(printversion);
        */

        var canvas = newplot.getCanvas();
        var img = canvas.toDataURL("img/png");

        $('div #'+panelname).data('img1', img);
        $('div #'+panelname).data('title1', title);
    }
}

function makePlotD2() {
    var panelname = $(this).parents("div .ui-widget-content").attr('id');
    var time_choice = $('#' + panelname + ' input[type=radio]:checked').val();
    var unit = $(this).attr('unit');
    var unit2 = $(this).attr('unit2');
    var title = $(this).attr('title');

    $.ajax({
        url: "cache/" + panelname + "." +  $(this).attr('id') + "." + time_choice + ".json",
        method: "post",
        dataType: "json",
        data: {time_choice: time_choice, id: $(this).attr('id')},
        success: plotdata2
    });

    function plotdata2(data) {
        var d0 = new Date(data["xmin"]*1000);
        var dmin = Date.UTC(d0.getUTCFullYear(), d0.getUTCMonth(), d0.getUTCDate(), 0, 0, 0);
        var dmax = dmin + 86400000;
        var hour = 3600000; 
        var ticks = [];
        for (var i=0; i<=24; i+=3) ticks.push(dmin + i*hour);

        var options_left = {
            canvas: true,
          series: { 
              lines: {
                  lineWidth: 1
              }, 
              shadowSize: 1.5, 
          },
          xaxis: { 
              mode: 'time',
              min: dmin, 
              max: dmax, 
              ticks: ticks,
              timeformat: "%H:%M",
              axisLabelUseCanvas: true,
              axisLabelPadding: 16,
              axisLabelFontSizePixels: 16,
              axisLabelFontFamily: 'Arial',
              axisLabelColour: 'black',
              axisLabel: 'UTC Time   on   ' + formatDate(new Date(dmin)),
          },
          yaxis: { 
              min: data["ymin"], 
              max: data["ymax"],
              labelWidth: 40,
              axisLabelUseCanvas: true,
              axisLabelFontFamily: 'Arial',
              axisLabel: '[ ' + unit + ' ]'
          },
          grid: { 
              margin: {} 
          },
          hooks: { 
              drawSeries: [fillShape] 
          }
        }

    var dataset=[];
    
    for (var i = 0; i < data['data'].length; i++) {
        for (var j = 0; j < data['data'][i]["path"].length; j++) {
            if (data['data'][i]["path"][j] != null)
                data['data'][i]["path"][j][0] *= 1000;
        }
        dataset.push({
            data: data['data'][i]["path"], 
            color: data['data'][i]["color"]});
    }

    function fillShape(plot, ctx, series) {

		var plotOffset = plot.getPlotOffset();
		var offset_x = plotOffset.left;
		var offset_y = plotOffset.top;

		function coord_x(x) {
			return offset_x + series.xaxis.p2c(x)
		};
		function coord_y(y) {
			return offset_y + series.yaxis.p2c(y)
		};

		var data = series.data;
		var chk = 0;

		ctx.beginPath();
		for (var i = 0; i < data.length; i++) {
			if (data[i] == null) {
				chk = 0;
				continue;
			}
			if (chk == 0) {
				ctx.moveTo(coord_x(data[i][0]), coord_y(data[i][1]));
				chk = 1;
			} else {
				ctx.lineTo(coord_x(data[i][0]), coord_y(data[i][1]));
			}
		}
		ctx.fillStyle = series.color;
		ctx.fill();
		ctx.closePath();
    }

    var colorbar = [];
    var myColors = [];

    var b = hex2rgb(data['data'][0]['color']);
    var a = hex2rgb(data['data'][data['data'].length - 1]['color']);

    myColors = genColors(a, b);

    var cntrArr = []; //last points
    var cntDataset = [];
    var tempObj = {
        data: [],
        color: "",
    };
    var ymin = data['data'][0]["layer"];
    var ymax = data['data'][data['data'].length - 1]["layer"];
    var ystep = (ymax-ymin)/50;
    console.log(ymin, ymax);
    

    for (var i = 0; i < 50; i++) { //on each line
        for (var j = 0; j < 3; j++) {
            cntrArr.push([j, ymin + ystep*i]);
        }
        tempObj.data = cntrArr;
        tempObj.color = myColors[i];

        cntDataset.push(tempObj);
        tempObj = {};
        cntrArr = [];
    }
        var options_right = {
            canvas: true,
          series: { 
              lines:{
                  fillColor: {colors: myColors},
                  lineWidth: 0,
                  fill: true,
              }, 
              shadowSize: 1.5, 
          },
          xaxis: {
              axisLabel:  unit2,
              axisLabelPadding: 16,
              tickLength: 0,
              ticks: [0, 1],
              tickColor: 'transparent',
              tickFormatter: function(val, axis) {
                  return ""
              },
              axisLabelUseCanvas: true,
              axisLabelFontSizePixels: 16,
              axisLabelFontFamily: 'Arial',
          },
          yaxis: { 
              min: ymin, 
              max: ymax,
              position: "right",
              axisLabelUseCanvas: true,
              ticks: 10
          },
          grid: { 
              margin: {left:0, right:0, bottom: 17} 
          },
          hooks: { 
          }
        }

        var plot1 = $.plot("#plot2d", dataset, options_left);
        var plot2 = $.plot("#plotColorBar", cntDataset, options_right);

        var canvas1 = plot1.getCanvas();
        var canvas2 = plot2.getCanvas();

        var canvas3 = document.createElement('canvas');
        canvas3.width = 800;
        canvas3.height = 400;
        var ctx3 = canvas3.getContext('2d');

        ctx3.drawImage(canvas1, 0, 10);
        ctx3.drawImage(canvas2, 700, 10);

        var img = canvas3.toDataURL("img/png");
        $('div #'+panelname).data('img2', img);
        $('div #'+panelname).data('title2', title);
  }
}

