#!/usr/bin/env python
import numpy as np
import simplejson as json
import ConfigParser
import re
import sys
import cgi, cgitb
import adeitools as adei
import matplotlib as mpl
import matplotlib.cm as cm

mpl.use('Agg')
import matplotlib.pyplot as plt

c1 = re.compile('^sensor\d+')
c2 = re.compile('^server')

try:
    config = ConfigParser.ConfigParser()
    config.read("../hatpro.ini")
    sensors = [dict(config._sections[a]) 
            for a in config.sections() if c1.match(a)]
except:
    print "fail to open config file"
    sys.exit()

try:
    config = ConfigParser.ConfigParser()
    config.read("../server.ini")
    servers = [dict(config._sections[a]) 
            for a in config.sections() if c2.match(a)]
except:
    print "fail to open config file"
    sys.exit()


# initialize adei server
try:
    adeiservers = {
            servers[i]['__name__']: adei.adeiserver(
                servers[i]['db_name'], 
                servers[i]['db_server'], 
                servers[i]['db_host']) 
            for i in range(len(servers)) }
    for s in adeiservers:
        adeiservers[s].updateSensor('name', 'column')
except:
    print "fail to initialize server"
    sys.exit()


# select sensor
for sensor in sensors:
    if sensor['method'] != 'hatpro_content':
        continue

    print "device name:              %s" % sensor['device']
    print "server name:              %s" % sensor['server']
    print "group name:               %s" % sensor['group']
    print "sensor name:              %s" % sensor['sensor']
    srv = adeiservers[sensor['server']]

    # sensor group time stamps
    t0 = adei.secSinceEpoch(srv.groupStamp(sensor['group']))         # last time stamp
    t00 = adei.secSinceEpoch(srv.groupLastDay(sensor['group']))      # beginning of current day
    t01 = t00 - 86400                                                # beginning of the day before
    t02 = t00 - 86400*2                                              # beginning of the second day before
    newstamp = t0
    daystamp = t00
    print "time stamps (new/day):    %s/%s" % (newstamp, daystamp)

    sensorList = srv.sensorFilter(sensor['sensor'])
    colormap = cm.GnBu

    for timekey in ['0d']:
    #for timekey in ['0d', '1d', '2d']:
        filename = "../cache/%s.%s.PRF.%s.json" % (sensor['device'], sensor['sensor'], timekey)

        # get stamp from old json file and decide update_level
        # if update_level is 0, no update
        # if update_level is 1, update for '0d'
        # if update_level is 2, update for all ('0d', '1d', '2d')
        try:
            with open(filename, 'r') as f:
                oldstamp = json.loads(f.read())[0]['stamp']
        except:
            oldstamp = 0
        update_level = 0
        if oldstamp < newstamp: update_level += 1 
        if oldstamp < daystamp: update_level += 1 
        print "outfile (update level):   %s (%s)" % (filename,update_level)

        # get data
        dt = 86400
        data = None

        if timekey == '0d':# and update_level >= 1:
            time = srv.getSensorTime(sensorList[0], t00, dt)
            elevation = srv.getSensorData(srv.sensorFilter('L2A.ELEVATION.ANGLE')[0], t00, dt)
            azimuth = srv.getSensorData(srv.sensorFilter('L2A.AZIMUTH.ANGLE')[0], t00, dt)
            data = [ srv.getSensorData(i, t00, dt) for i in sensorList ]

            #azimuth = [azimuth[i] for i in range(len(elevation)) if elevation[i] != '90']
            #time = [time[i] for i in range(len(elevation)) if elevation[i] != '90']
            #data = [data[0][i] for i in range(len(elevation)) if elevation[i] != '90']
            #elevation = [elevation[i] for i in range(len(elevation)) if elevation[i] != '90']

            for a, b, c, d  in zip(elevation[:-1], elevation[1:], azimuth[:-1], azimuth[1:]):
                if c != d:
                    print a, b, c, d

            #print time
            #print data
            #print elevation
            #print azimuth
        if timekey == '1d' and update_level == 2:
            time = srv.getSensorTime(sensorList[0], t01, dt)
            data = [ srv.getSensorData(i, t01, dt) for i in sensorList ]
        if timekey == '2d' and update_level == 2:
            time = srv.getSensorTime(sensorList[0], t02, dt)
            data = [ srv.getSensorData(i, t02, dt) for i in sensorList ]

        if data:
            # contour
            ys = range(len(data))
            cs = plt.contourf(time, ys, data, 50, cmap=colormap)

            # output
            pathList = []
            pathList.append({"xmin": time[0], "xmax": time[-1], "ymin": ys[0], "ymax": ys[-1], "stamp": newstamp})

            for l, p, c in zip(cs.layers, cs.collections, cs.tcolors):
                color = mpl.colors.rgb2hex(c[0][:3])
                for path in p.get_paths():
                    pp = []
                    for vertex, code in zip(path.vertices, path.codes):
                        if len(pp) != 0 and code == 1:
                            pp.append(None)
                            pp.append(vertex.tolist())
                        else:
                            pp.append(vertex.tolist())

                    pathList.append({"layer": l, "path": pp, "color": color})

            try:
                with  open(filename, "w") as f:
                    json.dump(pathList, f)
            except:
                pass
