#!/usr/bin/env python
import sys
import cgi, cgitb
#cgitb.enable()

import numpy as np
import simplejson as json

### import matplotlib for web
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.cm as cm
import matplotlib.pyplot as plt

### load configuation and modify settings
import adeiconfig as config
config.DB_NAME = 'kitcube2'
### then import adeitools
import adeitools as adei

#sys.stderr = open("/home/chuan/projects/kitcube/kitcube-status/test/adeitools/cgi-error.log", "a")

data_group = config.HATPRO_GRP_C
data_field = config.HATPRO_FLD_C_5
day = 10

def main():
    ### header
    sys.stdout.write("Content-Type: application/json\n\n")

    ### set time to lastday
    lastday = adei.lastDay(data_group)
    lastday_sec = adei.secSinceEpoch(lastday) - 86400*day

    ### get data
    maskList = adei.getMaskList(data_field, data_group)
    data = adei.getDataByMask(maskList, data_group, lastday_sec, 86400)

    ### generate contour paths
    #colormap = cm.RdBu_r
    colormap = cm.GnBu_r
    yr, tr = data.shape

    cs = plt.contourf(range(tr), range(yr), data, 50, cmap=colormap)

    ### generate path list
    pathList = []

    ### ### axis range
    pathList.append({"xmin": 0, "xmax": tr-1, "ymin": 0, "ymax": yr-1})

    ### ### contours
    for l, p, c in zip(cs.layers, cs.collections, cs.tcolors):
        color = mpl.colors.rgb2hex(c[0][:3])
        for path in p.get_paths():
            pp = []
            for vertex, code in zip(path.vertices, path.codes):
                if len(pp) != 0 and code == 1:
                    pp.append(None)
                    pp.append(vertex.tolist())
                else:
                    pp.append(vertex.tolist())

            pathList.append({"layer": l, "path": pp, "color": color})

    ### json output
    sys.stdout.write(json.dumps(pathList))

    ### close
    sys.stdout.close()


if __name__ == "__main__":
    main()
